using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using SimpleJSON;
using MVR.FileManagementSecure;
using UnityEngine.UI;

namespace JayJayWon
{
    public class ForcePersonCache : MVRScript
    {

        SuperController.OnAtomUIDRename atomRenameHandle;
        SuperController.OnAtomRemoved atomRemoveHandle;
        SuperController.OnAtomAdded atomAddedHandle;
        SuperController.OnSceneLoaded sceneLoadedHandle;
        public override void Init()
        {
            try
            {
                if (containingAtom.name == "CoreControl" && containingAtom.type == "SessionPluginManager")
                {
                    atomRenameHandle = new SuperController.OnAtomUIDRename(this.OnAtomNameUpdate);
                    SuperController.singleton.onAtomUIDRenameHandlers += atomRenameHandle;

                    atomRemoveHandle = new SuperController.OnAtomRemoved(this.OnAtomRemoved);
                    SuperController.singleton.onAtomRemovedHandlers += atomRemoveHandle;

                    sceneLoadedHandle = new SuperController.OnSceneLoaded(OnSceneLoaded);
                    SuperController.singleton.onSceneLoadedHandlers += sceneLoadedHandle; 

                    atomAddedHandle = new SuperController.OnAtomAdded(OnAtomAdded);
                    SuperController.singleton.onAtomAddedHandlers += atomAddedHandle;



                }
                else
                {
                    SuperController.LogMessage("ForcePersonCache must be loaded as a Session Plugin");
                    
                }
                JSONStorableString text = new JSONStorableString("text", "ForcePersonCache is a simple plugin that speeds up the it takes to create an initial person atom. It should be installed as a Session Plugin and set as User Default in Session Plugin Presets (see the VAM help text under Session Plugin Presets for more details). This will ensure the plugin runs when VAM first loads and on any Hard Reset.\n\nThe benefit of the plugin is that the next time a Person atom is created it will be much quicker. The down side is that the VAM start up process will take a little longer.");
                var textField = CreateTextField(text);
                textField.height = 500f;

                JSONStorableString creatorText = new JSONStorableString("creatorText", "ForcePersonCache v1.0\nby JayJayWon\n\nIf you like this plugin, please support me on Patreon at https://www.patreon.com/JayJayWon");
                var textField2 = CreateTextField(creatorText,true);
                textField2.height = 500f;

            }
            catch (Exception e)
            {
                SuperController.LogError($"{nameof(ForcePersonCache)}.{nameof(Init)}: {e}");
            }
        }


        private void OnSceneLoaded()
        {
        }
        private void OnAtomAdded(Atom atom)
        {
        }

        private void OnAtomNameUpdate(string oldName, string newName)
        {

        }

        private void OnAtomRemoved(Atom atom)
        {

        }

        public void OnEnable()
        {
            try
            {

            }
            catch (Exception e)
            {
                SuperController.LogError($"{nameof(ForcePersonCache)}.{nameof(OnEnable)}: {e}");
            }
        }

        public void OnDisable()
        {
            try
            {

            }
            catch (Exception e)
            {
                SuperController.LogError($"{nameof(ForcePersonCache)}.{nameof(OnDisable)}: {e}");
            }
        }

        public void OnDestroy()
        {
            try
            {
                SuperController.singleton.onAtomUIDRenameHandlers -= atomRenameHandle;
                SuperController.singleton.onAtomRemovedHandlers -= atomRemoveHandle;
                SuperController.singleton.onAtomAddedHandlers -= atomAddedHandle;
                SuperController.singleton.onSceneLoadedHandlers -= sceneLoadedHandle;

            }
            catch (Exception e)
            {
                SuperController.LogError($"{nameof(ForcePersonCache)}.{nameof(OnDestroy)}: {e}");
            }
        }


        void Start()
        {
            foreach (Atom atom in SuperController.singleton.GetAtoms())
            {
                if (atom.category == "People" || atom.name =="Person") return;
            }
            StartCoroutine(CreateAtom( (atomName) =>
            {
                try
                {

                    Atom atom = SuperController.singleton.GetAtomByUid("Person");
                    atom.Remove();

                }
                catch (Exception e) { SuperController.LogError("Exception caught: " + e); }
            }));
        }
        private IEnumerator CreateAtom( Action<string> onAtomCreated)
        {

            yield return SuperController.singleton.AddAtomByType("Person", "Person");

            onAtomCreated("Person");
        }
        void Update()
        {
        }
    }

}
